<!-- 
 * Licensed Materials - Property of IBM
 * com.ibm.datatools.xquery.ui
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 *   Use, duplication or disclosure restricted 
 *   by GSA ADP Schedule Contract with IBM Corp.
 */
-->
<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/TR/REC-html40" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output indent="yes" method="xml" omit-xml-declaration="yes"/>
	
	<xsl:template match="/body">
		<xsl:element name="body" namespace="">

			<xsl:for-each select="xpath">
				<xsl:sort data-type="number" order="ascending" select="number(@level)"/>
				<xsl:variable name="uri" select="@uri"/>
				<xsl:variable name="fullpath" select="@fullpath"/>
				<xsl:variable name="currparentabspath" select="@parentabspath"/>
				<!--  starts-with: true if the first string starts with the second-->
				<!-- the current path (see for loop) starts with another one (xpath and parentabspath) -->
				<!-- we're storing the path beginning in a variable -->
				<!--  the starts-with for parentabspath prevents a mixture of nested fors-->
				<xsl:variable name="xqueryvariable" select="//xpath[@uri=$uri and @variablename!=&apos;&apos; and  starts-with($currparentabspath,@parentabspath)   and starts-with($fullpath,@fullpath)   ]"/>
 				<xsl:variable name="siblingvariable" select="//xpath[@uri=$uri and @variablename!=&apos;&apos; and $currparentabspath=@parentabspath   and $fullpath=@fullpath]"/>
				
				<xsl:choose>
					<xsl:when test="count($xqueryvariable)&gt;1 and (@variabletype=&apos;for&apos; or @variabletype=&apos;let&apos;)">
						<xsl:variable name="targetvariable" select="$xqueryvariable[count($xqueryvariable)-1]"/>
						<xsl:element name="xpath" namespace="">
							<xsl:attribute name="name" namespace="">
								<xsl:value-of select="$targetvariable/@variablename"/>
							</xsl:attribute>
							<xsl:attribute name="type" namespace="">
								<xsl:value-of select="@variabletype"/>
							</xsl:attribute>
							<xsl:attribute name="fullpath" namespace="">
								<xsl:value-of select="$fullpath"/>
							</xsl:attribute>
							<xsl:attribute name="relativepath" namespace="">
								$<xsl:value-of select="$targetvariable/@variablename"/>
								<xsl:value-of select="substring-after($fullpath,$targetvariable/@fullpath)"/>
							</xsl:attribute>
							<xsl:attribute name="modelpath" namespace="">
								<xsl:value-of select="@modelabspath"/>
							</xsl:attribute>
						</xsl:element>
					</xsl:when>

					<xsl:when test="count($xqueryvariable)&gt;1">
						<xsl:variable name="targetvariable" select="$xqueryvariable[count($xqueryvariable)]"/>
 <!-- only differernce to case one -->
						<xsl:element name="xpath" namespace="">
							<xsl:attribute name="name" namespace="">
								<xsl:value-of select="$targetvariable/@variablename"/>
							</xsl:attribute>
							<xsl:attribute name="type" namespace="">
								<xsl:value-of select="@variabletype"/>
							</xsl:attribute>
							<xsl:attribute name="fullpath" namespace="">
								<xsl:value-of select="$fullpath"/>
							</xsl:attribute>
							<xsl:attribute name="relativepath" namespace="">
								$<xsl:value-of select="$targetvariable/@variablename"/>
								<xsl:value-of select="substring-after($fullpath,$targetvariable/@fullpath)"/>
							</xsl:attribute>
							<xsl:attribute name="modelpath" namespace="">
								<xsl:value-of select="@modelabspath"/>
							</xsl:attribute>
						</xsl:element>
					</xsl:when>

					<!-- i.e., all RETURNs with siblings-->
					<xsl:when test="@variabletype=&apos;return&apos; and  count($siblingvariable)&gt;0">
						<xsl:element name="xpath" namespace="">
							<xsl:attribute name="name" namespace="">
								<xsl:value-of select="$siblingvariable/@variablename"/>
							</xsl:attribute>
							<xsl:attribute name="type" namespace="">
								<xsl:value-of select="@variabletype"/>
							</xsl:attribute>
							<xsl:attribute name="fullpath" namespace="">
								<xsl:value-of select="$fullpath"/>
							</xsl:attribute>
							<xsl:attribute name="relativepath" namespace="">
								$<xsl:value-of select="$siblingvariable/@variablename"/>
							</xsl:attribute>
							<xsl:attribute name="modelpath" namespace="">
								<xsl:value-of select="@modelabspath"/>
							</xsl:attribute>
						</xsl:element>
					</xsl:when>
					
					<!-- all the paths (not in a FOR) that have exact one beginning -->
					<xsl:when test="count($xqueryvariable)=1 and  @variabletype!=&apos;for&apos;">
						<xsl:variable name="targetvariable" select="$xqueryvariable[count($xqueryvariable)]"/>
						<xsl:element name="xpath" namespace="">
							<xsl:attribute name="name" namespace="">
								<xsl:value-of select="$targetvariable/@variablename"/>
							</xsl:attribute>
							<xsl:attribute name="type" namespace="">
								<xsl:value-of select="@variabletype"/>
							</xsl:attribute>
							<xsl:attribute name="fullpath" namespace="">
								<xsl:value-of select="$fullpath"/>
							</xsl:attribute>
							<xsl:attribute name="relativepath" namespace="">
								$<xsl:value-of select="$targetvariable/@variablename"/>
								<xsl:value-of select="substring-after($fullpath,$targetvariable/@fullpath)"/>
							</xsl:attribute>
							<xsl:attribute name="modelpath" namespace="">
								<xsl:value-of select="@modelabspath"/>
							</xsl:attribute>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>
	
</xsl:stylesheet>
