<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/TR/REC-html40" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<!--
	Licensed Materials - Property of IBM
	Plugin: com.ibm.datatools.xquery.ui
	
	(C) Copyright IBM Corporation 2005. All Rights Reserved.
	  Note to U.S. Government Users Restricted Rights:
	  Use, duplication or disclosure restricted 
	  by GSA ADP Schedule Contract with IBM Corp.
	*************************************************** 
	 
	 This XSLT script does the actual transformation of the XQuery builder XML model
	 to the resulting XQuery. All steps run in JAVA code before manipulate the XML model
	 itself, since there are some checks and modifications that cannot be done in XSLT.
	 
	 The result of the transformation is the XQuery embedded in formatting tags: line, block.
	 -->
 	<xsl:param name="i18n-doc">no-value-passed</xsl:param>
 	<xsl:output indent="no" method="xml" omit-xml-declaration="yes"/>
	<!-- ********************************************** -->
	<xsl:variable name="querytype">sql</xsl:variable><!-- check whether it makes sense to keep it !!! -->
 	<xsl:variable name="accessor_start">db2-fn:xmlcolumn("</xsl:variable>
	<xsl:variable name="accessor_end">")</xsl:variable>
	<xsl:variable name="root_identifier">-doc-root-</xsl:variable>
	<xsl:variable name="i18n-strings" select="$i18n-doc//string"/>
	<!-- ********************************************** -->
	<xsl:template match="*[1]">
		<xsl:element name="body" namespace="">
			<xsl:choose>
				<xsl:when test="local-name()=&apos;query&apos;">
					<xsl:call-template name="dotoplevel"/>
				</xsl:when>
				<xsl:otherwise> <!-- any other node than the query itself -->
					<xsl:call-template name="doone"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>	
	<!-- ********************************************** -->
	<xsl:template name="dotoplevel"> <!-- applied to the query node -->
		<xsl:call-template name="dosqlhead"/>
		<xsl:element name="block" namespace="">
			<xsl:if test="count(./prolog/description/text())!=0 and normalize-space(./prolog/description/text())!=''">
				<xsl:element name="line" namespace=""/>
				(: <xsl:value-of select="./prolog/description/text()"/> :)
			</xsl:if>
			<xsl:for-each select="./prolog/property[@name='preserve_white_space' and @value!='']">
				<xsl:element name="line" namespace=""/>
				declare boundary-space <xsl:if test="@value=&apos;true&apos;">preserve</xsl:if><xsl:if test="@value=&apos;false&apos;">strip</xsl:if>;
			</xsl:for-each>
			<xsl:for-each select="./prolog/namespace[@uri!='']">
				<xsl:element name="line" namespace=""/>
				declare namespace <xsl:value-of select="@prefix"/>="<xsl:value-of select="@uri"/>";
			</xsl:for-each>
			<xsl:if test="count(./prolog/property[@name='preserve_doc_order' and @value='false'])>0">
				<xsl:element name="line" namespace=""/>unordered {
			</xsl:if>
			<xsl:for-each select="./body/*[1]">
				<xsl:element name="line" namespace=""/>
				<xsl:call-template name="dovariablereturn"/>
			</xsl:for-each>
			<xsl:element name="line" namespace=""/>
			<xsl:if test="count(./prolog/property[@name='preserve_doc_order' and @value='false'])>0">
				}<xsl:element name="line" namespace=""/>
			</xsl:if>
		</xsl:element>
		<xsl:call-template name="dosqlfoot"/>			
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dovariablereturn">	<!-- applied to the body/* and sequences of returns -->
		<xsl:param name="returntype"/>					
		<xsl:choose>
			<!-- =================================== -->
			<xsl:when test="local-name()=&apos;variable&apos;">
				<xsl:call-template name="dovariable"/>
				<xsl:element name="line" namespace=""/>
				return (
				<xsl:element name="line" namespace=""/>
				<xsl:element name="block" namespace="">
					<xsl:if test="$returntype=''">
						<xsl:for-each select="./following-sibling::*[1]">
							<xsl:call-template name="dovariablereturn"/>
						</xsl:for-each>
					</xsl:if>
					<xsl:if test="$returntype='then'">
						<xsl:for-each select="(./parent::*/following-sibling::return[@type='then']/*)[1]">
							<xsl:call-template name="dovariablereturn">
								<xsl:with-param name="returntype">then</xsl:with-param>
							</xsl:call-template>
						</xsl:for-each>
					</xsl:if>
					<xsl:if test="$returntype='else'">
						<xsl:for-each select="(./parent::*/following-sibling::return[@type='else']/*)[1]">
							<xsl:call-template name="dovariablereturn">
								<xsl:with-param name="returntype">else</xsl:with-param>
							</xsl:call-template>
						</xsl:for-each>
					</xsl:if>
					<xsl:if test="$returntype='return'">
						<xsl:for-each select="(./parent::*/following-sibling::return[@type='return']/*)[1]">
							<xsl:call-template name="dovariablereturn">
								<xsl:with-param name="returntype">return</xsl:with-param>
							</xsl:call-template>
						</xsl:for-each>
					</xsl:if>
				</xsl:element>)
				<xsl:element name="line" namespace=""/>
			</xsl:when>
			<!-- =================================== -->
			<xsl:otherwise>
				<xsl:call-template name="doone"/>
				<xsl:variable name="isxquerycomment" select="(local-name()='rawtext' and @type='xquerycomment')"/>
				<xsl:if test="not($isxquerycomment) and count(following-sibling::*[not(local-name()='rawtext' and @type='xquerycomment')])>0">,</xsl:if>
				<xsl:if test="$returntype=''">
					<xsl:for-each select="./following-sibling::*[1]">
						<xsl:element name="line" namespace=""/>
						<xsl:call-template name="dovariablereturn"/>
					</xsl:for-each>
					<xsl:if test="count(./following-sibling::*)=0"><xsl:element name="line" namespace=""/></xsl:if>
				</xsl:if>
				<xsl:if test="$returntype='then'">
					<xsl:for-each select="(./parent::*/following-sibling::return[@type='then']/*)[1]">
						<xsl:if test="not(local-name()='rawtext' and @type='xquerycomment')">,</xsl:if>
						<xsl:element name="line" namespace=""/>
						<xsl:call-template name="dovariablereturn">
							<xsl:with-param name="returntype">then</xsl:with-param>
						</xsl:call-template>
					</xsl:for-each>
					<xsl:if test="count(./parent::*/following-sibling::return[@type='then']/*)=0"><xsl:element name="line" namespace=""/></xsl:if>
				</xsl:if>
				<xsl:if test="$returntype='else'">
					<xsl:for-each select="(./parent::*/following-sibling::return[@type='else']/*)[1]">
						<xsl:if test="not(local-name()='rawtext' and @type='xquerycomment')">,</xsl:if>
						<xsl:element name="line" namespace=""/>
						<xsl:call-template name="dovariablereturn">
							<xsl:with-param name="returntype">else</xsl:with-param>
						</xsl:call-template>
					</xsl:for-each>
					<xsl:if test="count(./parent::*/following-sibling::return[@type='else']/*)=0"><xsl:element name="line" namespace=""/></xsl:if>
				</xsl:if>
				<xsl:if test="$returntype='return'">
					<xsl:for-each select="(./parent::*/following-sibling::return[@type='return']/*)[1]">
						<xsl:if test="not(local-name()='rawtext' and @type='xquerycomment')">,</xsl:if>
						<xsl:element name="line" namespace=""/>
						<xsl:call-template name="dovariablereturn">
							<xsl:with-param name="returntype">return</xsl:with-param>
						</xsl:call-template>
					</xsl:for-each>
					<xsl:if test="count(./parent::*/following-sibling::return[@type='return']/*)=0"><xsl:element name="line" namespace=""/></xsl:if>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ****************** SQL/XML head **************** -->
	<xsl:template name="dosqlhead">
		values(XMLQUERY('
	</xsl:template>
	<!-- ****************** SQL/XML foot **************** -->
	<xsl:template name="dosqlfoot">
		' <xsl:if test="count(/query/prolog/documents/document[@uri!='' and @column='' and @usedfor!=''])!=0">
			PASSING BY REF
			<xsl:for-each select="/query/prolog/documents/document[@uri!='' and @column='' and @usedfor!='']">
				<xsl:sort data-type="text" order="ascending" select="@uri"/>
				<xsl:if test="position()!=1">, </xsl:if>
				CAST(? as XML) AS "<xsl:value-of select="@id"/>"
			</xsl:for-each>
		</xsl:if>		
		RETURNING SEQUENCE))
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dochildren">
		<xsl:for-each select="*">
			<xsl:call-template name="doone"/>
		</xsl:for-each>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doone">
		<xsl:choose>
			<xsl:when test="local-name()=&apos;statement&apos;">
				<xsl:call-template name="dostatement"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;quantifiedexpression&apos;">
				<xsl:call-template name="doquantifiedexpression"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;pi&apos;">
				<xsl:call-template name="dopi"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;function&apos;">
				<xsl:call-template name="dofunction"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;attribute&apos;">
				<xsl:call-template name="doattribute"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;function&apos;">
				<xsl:call-template name="dofunction"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;expression&apos;">
				<xsl:call-template name="doexpression"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;element&apos;">
				<xsl:call-template name="doelement"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;rawtext&apos;">
				<xsl:call-template name="dorawtext"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;xpath&apos;">
				<xsl:call-template name="doxpath"></xsl:call-template>
			</xsl:when>
			<xsl:when test="local-name()=&apos;return&apos;">
				<xsl:call-template name="doreturn"/>
			</xsl:when>
			<xsl:when test="local-name()=&apos;variable&apos;">
				<xsl:call-template name="dovariable"/>
			</xsl:when>
			<xsl:otherwise></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doxpath">
		<xsl:variable name = "uri" select="@uri"/>
		<xsl:if test="substring(@filter, string-length(@filter)-4, 5)='#true'">
			fn:distinct-values (
		</xsl:if>
		<xsl:if test="(name(..)='element' or name(..)='attribute') and contains(@relativepath, '/@') and not(contains(substring-after(@relativepath, '/@'), '/'))">
			text {
		</xsl:if>
		<xsl:variable name="relativepath">
		<xsl:choose>
			<xsl:when test="@name='#document'">-doc-root-</xsl:when>
			<xsl:otherwise><xsl:value-of select="@relativepath"/></xsl:otherwise>
		</xsl:choose>
			
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="@relativepath=&apos;&apos;">
				(: <xsl:value-of select="$i18n-strings[@key='err_no_rel_path']"/> :)
			</xsl:when>
			<xsl:when test="$querytype='sql' and starts-with(@relativepath, $root_identifier) and count(/query/prolog/documents/document[@uri=$uri and @column=''])!=0">
				$<xsl:value-of select="./ancestor::query/prolog/documents/document[@uri=$uri]/@id"/>
				<xsl:value-of select="substring-after($relativepath, $root_identifier)"/>
			</xsl:when>
			<xsl:when test="starts-with(@relativepath, $root_identifier) and ./ancestor::query/prolog/documents/document[@uri=$uri]/@column!=''">
				<xsl:value-of select="$accessor_start"/>
				<xsl:if test="./ancestor::query/prolog/documents/document[@uri=$uri]/@omitschema='N'">
					<xsl:value-of select="./ancestor::query/prolog/documents/document[@uri=$uri]/@schema"/><xsl:text>.</xsl:text>
				</xsl:if>
				<xsl:value-of select="./ancestor::query/prolog/documents/document[@uri=$uri]/@table"/><xsl:text>.</xsl:text>
				<xsl:value-of select="./ancestor::query/prolog/documents/document[@uri=$uri]/@column"/>
				<xsl:value-of select="$accessor_end"/>
				<xsl:value-of select="substring-after($relativepath, $root_identifier)"/>
			</xsl:when>
			<xsl:when test="starts-with(@relativepath, $root_identifier)">
				<xsl:value-of select="$accessor_start"/>
				<xsl:value-of select="./@uri"/>
				<xsl:value-of select="$accessor_end"/>
				<xsl:value-of select="substring-after($relativepath, $root_identifier)"/>
			</xsl:when>
			<xsl:when test="$relativepath='-doc-root-'">
				<xsl:value-of select="@relativepath"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$relativepath"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="count(expression)&gt;0">
			<xsl:call-template name="doxpathexpression"/>
		</xsl:if>
		<!-- compute whether to append /text() respectively /fn:string() to the path  -->
		<!-- there is also handleRawtextString() in class XQueryPreprocessor that could be used-->
		<!-- there is also handleFlwor() in class XQueryPreprocessor that could be used-->
		<xsl:variable name="a" select="./ancestor::expression/rawtext"/>
		<xsl:variable name="b" select="concat(substring(translate($a,' &#9;&#10;&#13;',''),1,1),substring-after($a,substring(translate($a,' &#9;&#10;&#13;',''),1,1)))"/><!-- trim the raw text -->
		<xsl:variable name="c" select="./ancestor::expression/xpath"/>
		<xsl:variable name="d" select="./following-sibling::* | ./preceding-sibling::xpath"/>
		<!-- comparison with a rawtext -->
		<xsl:if test="count($a) = 1  and @isleaf='false' and starts-with($b, '&quot;') and substring($b, string-length($b), 1)='&quot;' and substring(@relativepath, string-length(@relativepath)-5, 6) != 'text()'">
			<xsl:text>/text()</xsl:text>
		</xsl:if>
		<!-- comparison with another XPath -->
		<xsl:if test="count($c) = 2  and count($d) = 1">
			<xsl:text>/fn:string(.)</xsl:text>
		</xsl:if>
		<xsl:if test="(name(..)='element' or name(..)='attribute') and contains(@relativepath, '/@') and not(contains(substring-after(@relativepath, '/@'), '/'))">
			}
		</xsl:if>
		<xsl:if test="substring(@filter, string-length(@filter)-4, 5)='#true'">
			)
		</xsl:if>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doxpathexpression">
		<xsl:variable name="childcount" select="count(expression)"/>
		<xsl:if test="$childcount&gt;0">
			[ <xsl:for-each select="*">
				<xsl:value-of select="@openparanth"/>
				<xsl:for-each select="*">
					<xsl:choose>
						<xsl:when test="local-name()=&apos;function&apos;">
							<xsl:call-template name="dofunction"/>
						</xsl:when>
						<xsl:when test="local-name()=&apos;xpath&apos;">
							<xsl:call-template name="doxpath"/>
						</xsl:when>
						<xsl:otherwise> <!-- raw text -->
							<xsl:value-of disable-output-escaping="no" select="text()"/>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:if test="position()=1 and not(starts-with(../@operation, 'position'))">
						<xsl:text> </xsl:text>
						<xsl:value-of disable-output-escaping="no" select="../@operation"/>
						<xsl:text> </xsl:text>
					</xsl:if>
					<xsl:if test="position()=1 and starts-with(../@operation, 'position')">/position()<xsl:value-of select="substring(../@operation, 9)"/></xsl:if>
				</xsl:for-each>
				<xsl:value-of select="@closeparanth"/>
				<xsl:if test="position()!=$childcount">
					<xsl:text> </xsl:text>
					<xsl:if test="@conjunction=&apos;&apos;">and</xsl:if>
					<xsl:value-of disable-output-escaping="no" select="@conjunction"/>
					<xsl:text> </xsl:text>
				</xsl:if>
			</xsl:for-each> ]
		</xsl:if>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dovariable">
		<xsl:choose>
			<xsl:when test="@type=&apos;for&apos; and count(*)=1 and (count(xpath)=1 or count(statement)=1 or count(function)=1 or count(rawtext)=1 or count(element)=1 or count(pi)=1 )">
				for <xsl:call-template name="dovariablename"/> in 
			</xsl:when>
			<xsl:when test="@type=&apos;let&apos; and count(*)=1 and (count(xpath)=1 or count(statement)=1 or count(function)=1 or count(rawtext)=1 or count(element)=1 or count(pi)=1 or count(quantifiedexpression)=1 )">
				let <xsl:call-template name="dovariablename"/> := 
			</xsl:when>
			<xsl:when test="@type=&apos;quantified&apos;">
				<xsl:call-template name="dovariablename"/> in
			</xsl:when>
			<xsl:when test="@type=&apos;userdefined&apos;">
				let <xsl:call-template name="dovariablename"/> := 
			</xsl:when>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="count(./*)=0">
			(: <xsl:value-of select="$i18n-strings[@key='err_empty_var']"/> :)
			</xsl:when>
			<xsl:when test="local-name(./*)!=&apos;xpath&apos;">
				<xsl:element name="block">
					<xsl:element name="line"/>
					<xsl:call-template name="dochildren"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="dochildren"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dovariablename">
		<xsl:choose>
			<xsl:when test="@name!=''">
				$<xsl:value-of select="@name"/>
			</xsl:when>
			<xsl:otherwise>
				<!-- here we could put a handling for non existing variable names -->
				<!-- add this back in next time -->
			<!-- 	(: no variable name defined :)  -->
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doreturn">
		<xsl:call-template name="dochildren"/>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doexpression">
		<xsl:choose>
			<xsl:when test="(local-name(..)=&apos;statement&apos;) and ( ../@type=&apos;flwor&apos;  or ../@type=&apos;condition&apos; )">
				<xsl:value-of select="@openparanth"/>
				<xsl:for-each select="*">
					<xsl:if test="local-name()=&apos;statement&apos;">(</xsl:if>
					<xsl:call-template name="doone"/>
					<xsl:if test="local-name()=&apos;statement&apos;">)</xsl:if>
					<xsl:if test="position()=1 and starts-with(../@operation, 'position')">/position()<xsl:value-of select="substring(../@operation, 9)"/></xsl:if>
					<xsl:if test="position()=1 and not(starts-with(../@operation, 'position'))">
						<xsl:text> </xsl:text><xsl:value-of disable-output-escaping="no" select="../@operation"/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:value-of select="@closeparanth"/>
			</xsl:when>
			<xsl:when test="local-name(..)=&apos;statement&apos; and ../@type=&apos;arithmetic&apos;">
				<xsl:value-of select="@openparanth"/>
				<xsl:call-template name="dochildren"/>
				<xsl:value-of select="@closeparanth"/>
				<xsl:element name="line"/>				
			</xsl:when>
			<xsl:when test="local-name(..)=&apos;quantifiedexpression&apos;">
				<xsl:value-of select="@openparanth"/>
				<xsl:call-template name="dochildren"/>
				<xsl:value-of select="@closeparanth"/>
				<xsl:text> </xsl:text>
				<xsl:if test="position()!=count(./../expression)">
					<xsl:value-of select="@operation"/>
				</xsl:if>
				<xsl:text> </xsl:text>
			</xsl:when>
			<xsl:otherwise></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dorawtext">
		<xsl:choose>
			<xsl:when test="@type=&apos;string&apos;">
				<xsl:value-of select="text()"/>
			</xsl:when>
			<xsl:when test="@type=&apos;xmlcomment&apos;">
				&lt;!-- <xsl:value-of select="text()"/> --&gt;
			</xsl:when>
			<xsl:when test="@type=&apos;xquerycomment&apos;">
				(: <xsl:value-of select="text()"/> :)
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="text()"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doelement">
		<xsl:choose>
			<!-- DIRECT element with NO content-->
			<xsl:when test="count(./*[local-name()!=&apos;name&apos; and
			                          local-name()!=&apos;attribute&apos; and
			                          local-name()!=&apos;namespace&apos;])=0 and
			                count(./name/rawtext)=1 and
			                count(./attribute/name/rawtext)=count(./attribute)">
				&lt;
				<xsl:if test="name/@prefix!=&apos;&apos;"><xsl:value-of select="name/@prefix"/><xsl:text>:</xsl:text></xsl:if>
				<xsl:value-of select="name/rawtext/text()"/>
				<xsl:call-template name="doattribute">
					<xsl:with-param name="inline">true</xsl:with-param>
				</xsl:call-template>
				<xsl:call-template name="donamespace"/>
				/&gt;
			</xsl:when>
			<!-- DIRECT element with content-->
			<xsl:when test="local-name(./name/*)=&apos;rawtext&apos;">
				&lt;<xsl:if test="name/@prefix!=&apos;&apos;"><xsl:value-of select="name/@prefix"/><xsl:text>:</xsl:text></xsl:if>
				<xsl:value-of select="name/rawtext/text()"/>
				<xsl:call-template name="doattribute">
					<xsl:with-param name="inline">true</xsl:with-param>
				</xsl:call-template>
				<xsl:call-template name="donamespace"/>
				&gt;
				<xsl:element name="block" namespace="">
					<xsl:element name="line" namespace=""/>
					<xsl:call-template name="doattribute">
						<xsl:with-param name="outline">true</xsl:with-param>
					</xsl:call-template>
					<xsl:for-each select="*[local-name()!=&apos;name&apos; and local-name()!=&apos;namespace&apos; and local-name()!=&apos;attribute&apos; ]">
						<xsl:element name="line" namespace=""/>
						<xsl:if test="local-name()!=&apos;element&apos;">{</xsl:if>
						<xsl:call-template name="doone"/>
						<xsl:if test="local-name()!=&apos;element&apos;">}</xsl:if>
				    </xsl:for-each>
					<xsl:element name="line" namespace=""/>
				</xsl:element>
				&lt;/<xsl:if test="name/@prefix!=&apos;&apos;"><xsl:value-of select="name/@prefix"/><xsl:text>:</xsl:text></xsl:if>
				<xsl:value-of select="name/rawtext/text()"/>&gt;
			</xsl:when>
			<!-- COMPUTED element with content-->
			<xsl:otherwise>
				element {
				<xsl:element name="line" namespace=""/>
				<xsl:element name="block" namespace="">
					<xsl:for-each select="name/*">
						<xsl:call-template name="doone"/>
					</xsl:for-each>
				</xsl:element>
				<xsl:element name="line" namespace=""/>
				} {
				<xsl:element name="block" namespace="">
					<xsl:element name="line" namespace=""/>
					<!-- ********************** namespaces -->
					<xsl:call-template name="donamespace">
						<xsl:with-param name="computed">true</xsl:with-param>
					</xsl:call-template>
					<xsl:if test="count(./namespace)>0 and (count(./namespace)-count(./*)!=0)">,<xsl:element name="line" namespace=""/></xsl:if>
					<!-- ********************** constructed attributes -->
					<xsl:call-template name="doattribute">
						<xsl:with-param name="alloutline">true</xsl:with-param>
					</xsl:call-template>
					<xsl:if test="count(./attribute)>0 and (count(./namespace)+count(./attribute)-count(./*)!=0)">,<xsl:element name="line" namespace=""/></xsl:if>
					<!-- ********************** existent attributes -->
					<xsl:for-each select="xpath[contains(@fullpath, &apos;/@&apos;)]">
						<xsl:if test="position()!=1">,<xsl:element name="line" namespace=""/></xsl:if>
						<xsl:call-template name="doone"/>
					</xsl:for-each>
					<xsl:if test="count(./xpath[contains(@fullpath, &apos;/@&apos;)])>0 and (count(./namespace)+count(./attribute)+count(./xpath[contains(@fullpath, &apos;/@&apos;)])-count(./*)!=0)">,<xsl:element name="line" namespace=""/></xsl:if>
					<!-- ********************** elements -->
					<xsl:for-each select="*[local-name()!=&apos;name&apos; and local-name()!=&apos;namespace&apos; and local-name()!=&apos;attribute&apos; and not(local-name()=&apos;xpath&apos; and contains(@fullpath, &apos;/@&apos;))]">
						<xsl:if test="position()!=1">,<xsl:element name="line" namespace=""/></xsl:if>
						<xsl:call-template name="doone"/>
					</xsl:for-each>
				</xsl:element>
				<xsl:element name="line" namespace=""/>
				}
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doattribute">		<!-- called from doelement only -->
		<xsl:param name="inline"/>			<!-- all that should be inlined (have a rawtext as name) - standard case -->
		<xsl:param name="outline"/>			<!-- all that should BUT CANNOT be inlined (anything else but a rawtext as name) - so outline them -->
		<xsl:param name="alloutline"/>		<!-- all in outline version (because they are in a computed element constructor) -->

		<xsl:for-each select="attribute">
			<xsl:choose>
				<xsl:when test="$inline=&apos;true&apos; and count(name/rawtext)=1">
				    <xsl:text> </xsl:text>
				    <xsl:if test="name/@prefix!=&apos;&apos;"><xsl:value-of select="name/@prefix"/><xsl:text>:</xsl:text></xsl:if>
					<xsl:value-of select="name/rawtext/text()"/>
					="{
					<xsl:for-each select="*[local-name()!=&apos;name&apos;]">
					    <xsl:call-template name="doone"/>
				    </xsl:for-each>
					}"
				</xsl:when>
				<xsl:when test="($outline=&apos;true&apos; and count(name/rawtext)!=1) or
				                 $alloutline=&apos;true&apos;">
					<xsl:if test="position()!=1">,<xsl:element name="line" namespace=""/></xsl:if>
				    attribute {
				    <xsl:element name="block" namespace="">
					    <xsl:element name="line" namespace=""/>
					    <xsl:if test="name/@prefix!=&apos;&apos; and count(name/rawtext)=1"><xsl:value-of select="name/@prefix"/><xsl:text>:</xsl:text></xsl:if>
						<xsl:for-each select="name/*">
						    <xsl:call-template name="doone"/>
					    </xsl:for-each>
					    <xsl:element name="line" namespace=""/>
				    </xsl:element>
				    } {
				    <xsl:element name="block" namespace="">
				        <xsl:element name="line" namespace=""/>
				    	<xsl:for-each select="*[local-name()!=&apos;name&apos;]">
						    <xsl:call-template name="doone"/>
					    </xsl:for-each>
					    <xsl:element name="line" namespace=""/>
				    </xsl:element>
				    }
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="donamespace">			<!-- called from doelement only -->
		<xsl:param name="computed"/>			<!-- indicates whether it's supposed to be a computed nsp -->
		<xsl:for-each select="namespace">
			<xsl:choose>
				<xsl:when test="$computed=&apos;true&apos;">
					<xsl:if test="position()!=1">,<xsl:element name="line"/></xsl:if>
					namespace <xsl:value-of select="@prefix"/> { <xsl:value-of select="@uri"/> }
				</xsl:when>
				<xsl:otherwise>
					<xsl:if test="position()!=1"><xsl:text> </xsl:text></xsl:if>
					xmlns:<xsl:value-of select="@prefix"/>="<xsl:value-of select="@uri"/>"
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dopi">
		<xsl:if test="count(name/rawtext)=1 and count(rawtext)=1">
			<xsl:text>&lt;?</xsl:text><xsl:value-of select="./name/rawtext"/>
			<xsl:text> </xsl:text>
			<xsl:for-each select="*[local-name()!=&apos;name&apos;]">
				<xsl:if test="position()!=1"><xsl:text> </xsl:text></xsl:if>
			    <xsl:call-template name="doone"/>
			</xsl:for-each>
			<xsl:text>?&gt;</xsl:text>
		</xsl:if>
		<xsl:if test="count(name/rawtext)!=1 or count(rawtext)!=1">
			processing-instruction 
			<xsl:element name="line" namespace=""/>
			<xsl:for-each select="name/*">
				<xsl:if test="position()!=1">,<xsl:element name="line" namespace=""/></xsl:if>
			    <xsl:call-template name="doone"/>
			</xsl:for-each>
			<xsl:element name="line" namespace=""/>
			 {
			<xsl:element name="line" namespace=""/>
			<xsl:for-each select="*[local-name()!=&apos;name&apos;]">
				<xsl:if test="position()!=1">,<xsl:element name="line" namespace=""/></xsl:if>
			    <xsl:call-template name="doone"/>
			</xsl:for-each>
			<xsl:element name="line" namespace=""/>
			}
		</xsl:if>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doquantifiedexpression">
		<xsl:value-of select="@condition"/>
		<xsl:for-each select="variable">
		<xsl:if test="position()>1">,<xsl:element name="line"/></xsl:if>
		    <xsl:call-template name="dovariable"/>
	    </xsl:for-each>
	    satisfies
		<xsl:for-each select="expression">
		    <xsl:call-template name="doexpression"/>
	    </xsl:for-each>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dofunction">
		<xsl:choose>
			<xsl:when test="@noarg=&apos;true&apos; and local-name(*[1])=&apos;xpath&apos;">
				<xsl:for-each select="xpath[1]">
					<xsl:call-template name="doxpath"/>/<xsl:value-of select="../@name"/>()
				</xsl:for-each>
			</xsl:when>
			<xsl:when test="@noarg=&apos;true&apos; and count(rawtext/*)=0 and count(*[local-name()!=&apos;rawtext&apos;])=0">       <!-- no parameter entered -->
				<xsl:value-of select="@name"/>()
			</xsl:when>
			<xsl:when test="@noarg=&apos;true&apos;">
				(<xsl:element name="block" namespace="">
					<xsl:element name="line" namespace=""/>
					<xsl:for-each select="*">
						<xsl:call-template name="doone"/>
					</xsl:for-each>
					<xsl:element name="line" namespace=""/>
				</xsl:element>
				)/<xsl:value-of select="@name"/>()
			</xsl:when>
			<xsl:otherwise>                                                         <!-- a regular function -->
				<xsl:value-of select="@name"/>
				(
				<xsl:variable name="minarg" select="@minarg"/>
				<xsl:for-each select="*">
					<xsl:choose>
						<xsl:when test="position() > $minarg and position()!=1">
							<xsl:choose>
								<xsl:when test="name()='rawtext' and string-length(text())=0"></xsl:when>
								<xsl:otherwise>,</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:when test="position() !=1">,</xsl:when>
					</xsl:choose>
					<xsl:call-template name="doone"/>
				</xsl:for-each>
				)
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="dostatement">
			<xsl:choose>
				<xsl:when test="@type=&apos;flwor&apos;">
					<xsl:call-template name="doflwor"/>
				</xsl:when>
				<xsl:when test="@type=&apos;condition&apos;">
					<xsl:call-template name="docondition"/>
				</xsl:when>
				<xsl:when test="@type=&apos;arithmetic&apos;">
					<xsl:variable name="expressioncount" select="count(expression)"/>
					<xsl:if test="$expressioncount&gt;0">
						<xsl:for-each select="expression">
							<xsl:call-template name="doone"/>
							<xsl:if test="position()!= $expressioncount">
								<xsl:text> </xsl:text><xsl:value-of select="@operation"/><xsl:text> </xsl:text>
							</xsl:if>
						</xsl:for-each>
					</xsl:if>
				</xsl:when>
				<xsl:when test="@type=&apos;sequence&apos;">
					<xsl:variable name="childcount" select="count(*)"/>
					<xsl:if test="$childcount&gt;0">
						<xsl:if test="$childcount&gt;1">
							(
								<xsl:element name="line" namespace=""/>
								<xsl:element name="block" namespace="">
								<xsl:for-each select="*">
									<xsl:if test="position()!=1 and not(name()='rawtext' and ./@type='xquerycomment')">,</xsl:if>
									<xsl:if test="position()!=1"><xsl:element name="line" namespace=""/></xsl:if>
									<xsl:call-template name="doone"/>
								</xsl:for-each>
								</xsl:element>
								<xsl:element name="line" namespace=""/>
							)
						</xsl:if>
						<xsl:if test="$childcount=1">
							<xsl:call-template name="dochildren"/>
						</xsl:if>
					</xsl:if>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="dochildren"/>
				</xsl:otherwise>
			</xsl:choose>
	</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="docondition">
		<xsl:variable name="expressioncount" select="count(expression)"/>
		if (
		<xsl:for-each select="expression">
			<xsl:if test="position()!=1">
				<xsl:element name="line" namespace=""/>
			</xsl:if>
			<xsl:call-template name="doone"/>
			<xsl:if test="position()!= $expressioncount">
				<xsl:choose>
					<xsl:when test="@conjunction=&apos;&apos;">
						and<xsl:text></xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text> </xsl:text>
						<xsl:value-of select="@conjunction"/>
						<xsl:text></xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>
		)<xsl:element name="line" namespace=""/>
		then (<xsl:element name="line" namespace=""/>
		<xsl:element name="block" namespace="">
			<xsl:for-each select="(return[@type=&apos;then&apos;]/*)[1]">
				<xsl:call-template name="dovariablereturn">
					<xsl:with-param name="returntype">then</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
		</xsl:element>
		)<xsl:element name="line" namespace=""/>
		else (<xsl:element name="line" namespace=""/>
		<xsl:element name="block" namespace="">
			<xsl:for-each select="(return[@type=&apos;else&apos;]/*)[1]">
				<xsl:call-template name="dovariablereturn">
					<xsl:with-param name="returntype">else</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
		</xsl:element>
		)</xsl:template>
	<!-- ********************************************** -->
	<xsl:template name="doflwor">
		<xsl:for-each select="variable[@type=&apos;for&apos;]">
			<xsl:call-template name="doone"/>
			<xsl:element name="line" namespace=""/>
		</xsl:for-each>
		<xsl:for-each select="variable[@type=&apos;let&apos; ]">
			<xsl:call-template name="doone"/>
			<xsl:element name="line" namespace=""/>
		</xsl:for-each>
		<xsl:variable name="expressioncount" select="count(expression)"/>
		<xsl:if test="$expressioncount&gt;0">
			where
			<xsl:for-each select="expression">
				<xsl:call-template name="doone"/>
				<xsl:if test="position()!= $expressioncount">
					<xsl:choose>
						<xsl:when test="normalize-space(@conjunction)=&apos;&apos;">
							and<xsl:text></xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text> </xsl:text>
							<xsl:value-of select="@conjunction"/>
							<xsl:text> </xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:element name="line" namespace=""/>
			</xsl:for-each>
		</xsl:if>
		<xsl:variable name="ordercount" select="count(orderby)"/>
		<xsl:if test="$ordercount&gt;0">
			order by
			<xsl:for-each select="orderby">
				<xsl:if test="position()!=1 and not(local-name(./*)='rawtext' and ./*/@type='xquerycomment')">
					<xsl:text>,</xsl:text>
					<xsl:element name="line" namespace=""/>
				</xsl:if>
				<xsl:choose>
					<xsl:when test="local-name(./*)=&apos;xpath&apos;">
						<xsl:call-template name="dochildren"/><xsl:text>[1]</xsl:text>
						<xsl:text> </xsl:text><xsl:value-of select="@direction"/>
					</xsl:when>
					<xsl:when test="local-name(./*)=&apos;statement&apos;">
						<xsl:text>(</xsl:text><xsl:call-template name="dochildren"/><xsl:text>)[1]</xsl:text>
						<xsl:text> </xsl:text><xsl:value-of select="@direction"/>
					</xsl:when>
					<xsl:when test="local-name(./*)='rawtext' and ./*/@type='xquerycomment'">
						<xsl:call-template name="dochildren"/>
					</xsl:when>
					<xsl:when test="local-name(./*)='rawtext' and ./*/@type='code'">
						<xsl:call-template name="dochildren"/>
						<xsl:text> </xsl:text><xsl:value-of select="@direction"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="dochildren"/>
						<xsl:text> </xsl:text><xsl:value-of select="@direction"/>
					</xsl:otherwise>
				</xsl:choose>				
			</xsl:for-each>
			<xsl:element name="line" namespace=""/>
		</xsl:if>
		<xsl:variable name="returncount" select="count(return)"/>
		<xsl:if test="$returncount&gt;0">
			return
			<xsl:choose>
				<xsl:when test="$returncount&gt;1">
					<xsl:element name="block" namespace="">
						<xsl:element name="line" namespace=""/>
						(
						<xsl:element name="line" namespace=""/>
						<xsl:for-each select="(return/*)[1]">
							<xsl:call-template name="dovariablereturn">
								<xsl:with-param name="returntype">return</xsl:with-param>
							</xsl:call-template>
						</xsl:for-each>
						<xsl:element name="line" namespace=""/>
						)
					</xsl:element>
				</xsl:when>
				<xsl:when test="local-name(./return/*)=&apos;element&apos;">
					<xsl:for-each select="return">
						<xsl:element name="line" namespace=""/>
						<xsl:element name="block" namespace="">
							<xsl:call-template name="doreturn"/>
						</xsl:element>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>
					<xsl:for-each select="return">
						<xsl:call-template name="doreturn"/>
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>

