<!-- 
 * Licensed Materials - Property of IBM
 * com.ibm.datatools.xquery.ui
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 *   Use, duplication or disclosure restricted 
 *   by GSA ADP Schedule Contract with IBM Corp.
 */
-->
<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/TR/REC-html40" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output indent="yes" method="xml" omit-xml-declaration="yes"/>
	<xsl:variable name="returnpath" select="/ibmxquerybuilderexpression/return/value/@path"/>
	<xsl:template match="/query/body">
		<xsl:element name="query" namespace="">
			<xsl:element name="body" namespace="">
				<xsl:call-template name="dochildren"/>
			</xsl:element>
		</xsl:element>
	</xsl:template>
															<!-- is applied to all body's children -->
	<xsl:template name="dochildren">
		<xsl:for-each select="*">
			<xsl:choose>
				<xsl:when test="local-name()=&apos;statement&apos;">
															<!-- statement -->
					<xsl:call-template name="dostatement"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;quantifiedexpression&apos;">
															<!-- quantified expressions -->
					<xsl:call-template name="doquantifiedexpression"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;pi&apos;">
															<!-- PI = processing instruction -->
					<xsl:call-template name="dopi"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;function&apos;">
															<!-- functions -->
					<xsl:call-template name="dofunction"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;attribute&apos;">
															<!-- atrributes -->
					<xsl:call-template name="doattribute"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;orderby&apos;">
															<!-- orderby-->
					<xsl:call-template name="doorderby"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;expression&apos;">
															<!-- expression-->
					<xsl:call-template name="doexpression"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;element&apos;">
															<!-- element-->
					<xsl:call-template name="doelement"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;rawtext&apos;">
															<!-- rawtext -->
					<xsl:call-template name="dorawtext"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;return&apos;">
															<!-- return-->
					<xsl:call-template name="doreturn"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;variable&apos;">
															<!-- varible-->
					<xsl:call-template name="dovariable"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;xpath&apos;">
															<!-- xpath-->
					<xsl:call-template name="doxpath"/>
				</xsl:when>
				<xsl:when test="local-name()=&apos;path&apos;">
															<!-- path == predicate -->
					<xsl:call-template name="dopath"/>
				</xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dopath">
															<!-- == predicate -->
		<xsl:for-each select="*">
															<!-- doxpath: fullpath, filter, namespace, prefix, type, relativepath, uri, dopath-->
			<xsl:element name="path" namespace="">
				<xsl:attribute name="closeparanth" namespace="">
					<xsl:value-of select="@closeparanth"/>
				</xsl:attribute>
				<xsl:attribute name="conjunction" namespace="">
					<xsl:value-of select="@conjunction"/>
				</xsl:attribute>
				<xsl:attribute name="openparanth" namespace="">
					<xsl:value-of select="@openparanth"/>
				</xsl:attribute>
				<xsl:attribute name="operation" namespace="">
					<xsl:value-of select="@operation"/>
				</xsl:attribute>
				<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
			</xsl:element>
		</xsl:for-each>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dovariable">
		<xsl:choose>
			<xsl:when test="count(*)=1 and xpath/@fullpath!=&apos;&apos;">
															<!-- if applied to one var and @fullpath not empty -->
				<xsl:element name="variable" namespace="">
					<xsl:attribute name="name" namespace="">
						<xsl:value-of select="@name"/>
					</xsl:attribute>
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="@type"/>
					</xsl:attribute>
					<xsl:element name="xpath" namespace="">
															<!-- doxpath calls dopath too -->
						<xsl:attribute name="fullpath" namespace="">
															<!-- doxpath: fullpath, filter, namespace, prefix, type, relativepath, uri, dopath-->
							<xsl:value-of select="xpath/@fullpath"/>
						</xsl:attribute>
						<xsl:attribute name="filter" namespace="">
							<xsl:value-of select="xpath/@filter"/>
						</xsl:attribute>
						<xsl:attribute name="namespace" namespace="">
							<xsl:value-of select="xpath/@namespace"/>
						</xsl:attribute>
						<xsl:attribute name="prefix" namespace="">
							<xsl:value-of select="xpath/@prefix"/>
						</xsl:attribute>
						<xsl:attribute name="type" namespace="">
							<xsl:value-of select="xpath/@type"/>
						</xsl:attribute>
						<xsl:attribute name="relativepath" namespace="">
							<xsl:value-of select="xpath/@relativepath"/>
						</xsl:attribute>
						<xsl:attribute name="uri" namespace="">
							<xsl:value-of select="xpath/@uri"/>
						</xsl:attribute>
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
															<!-- else -->
				<xsl:element name="variable" namespace="">
					<xsl:attribute name="name" namespace="">
						<xsl:value-of select="@name"/>
					</xsl:attribute>
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="@type"/>
					</xsl:attribute>
					<xsl:attribute name="uri" namespace="">
						<xsl:value-of select="xpath/@uri"/>
					</xsl:attribute>
					<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doxpath">
															<!-- fullpath, filter, namespace, prefix, type, relativepath, uri, dopath-->
		<xsl:element name="xpath" namespace="">
			<xsl:attribute name="fullpath" namespace="">
				<xsl:value-of select="@fullpath"/>
			</xsl:attribute>
			<xsl:attribute name="filter" namespace="">
				<xsl:value-of select="@filter"/>
			</xsl:attribute>
			<xsl:attribute name="namespace" namespace="">
				<xsl:value-of select="@namespace"/>
			</xsl:attribute>
			<xsl:attribute name="prefix" namespace="">
				<xsl:value-of select="@prefix"/>
			</xsl:attribute>
			<xsl:attribute name="type" namespace="">
				<xsl:value-of select="@type"/>
			</xsl:attribute>
			<xsl:attribute name="relativepath" namespace="">
				<xsl:value-of select="@relativepath"/>
			</xsl:attribute>
			<xsl:attribute name="uri" namespace="">
				<xsl:value-of select="@uri"/>
			</xsl:attribute>
			<xsl:call-template name="dopath"/>
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doreturn">
		<xsl:element name="return" namespace="">
			<xsl:attribute name="type" namespace="">
				<xsl:value-of select="@type"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doexpression">
		<xsl:element name="expression" namespace="">
			<xsl:attribute name="closeparanth" namespace="">
				<xsl:value-of select="@closeparanth"/>
			</xsl:attribute>
			<xsl:attribute name="conjunction" namespace="">
				<xsl:value-of select="@conjunction"/>
			</xsl:attribute>
			<xsl:attribute name="openparanth" namespace="">
				<xsl:value-of select="@openparanth"/>
			</xsl:attribute>
			<xsl:attribute name="operation" namespace="">
				<xsl:value-of select="@operation"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dorawtext">
		<xsl:element name="rawtext" namespace="">
			<xsl:attribute name="type" namespace="">
				<xsl:value-of select="@type"/>
			</xsl:attribute>
			<xsl:value-of select="text()"/>
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doelement">
		<xsl:element name="element" namespace="">
			<xsl:attribute name="name" namespace="">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:attribute name="namespace" namespace="">
				<xsl:value-of select="@namespace"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doattribute">
		<xsl:element name="attribute" namespace="">
			<xsl:attribute name="name" namespace="">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:attribute name="namespace" namespace="">
				<xsl:value-of select="@namespace"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dopi">
		<xsl:element name="pi" namespace="">
			<xsl:attribute name="name" namespace="">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:attribute name="namespace" namespace="">
				<xsl:value-of select="@namespace"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doquantifiedexpression">
		<xsl:element name="quantifiedexpression" namespace="">
			<xsl:attribute name="condition" namespace="">
				<xsl:value-of select="@condition"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dofunction">
		<xsl:element name="function" namespace="">
			<xsl:attribute name="name" namespace="">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doorderby">
		<xsl:element name="orderby" namespace="">
			<xsl:attribute name="direction" namespace="">
				<xsl:value-of select="@direction"/>
			</xsl:attribute>
			<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
		</xsl:element>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="dostatement">
		<xsl:choose>
			<xsl:when test="@type=&apos;flwor&apos;">
				<xsl:call-template name="doflwor"/>
															<!-- calls doFLOWR -->
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="statement" namespace="">
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="@type"/>
					</xsl:attribute>
					<xsl:call-template name="dochildren"/>
															<!-- calls doCHILDREN-->
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="doflwor">
															<!--  doFLWR: called from doSTATEMENT only -->
		<xsl:choose>
			<xsl:when test="count(*)=2 and variable/xpath/@fullpath = return/xpath/@fullpath">
															<!-- call-template passes the sequence of children as argument -->
				<xsl:element name="xpath" namespace="">
															<!-- make an XPath from a simple FLWOR -->
					<xsl:attribute name="fullpath" namespace="">
															<!-- doxpath:  fullpath, relativepath, filter, namespace, prefix, type, uri-->
						<xsl:value-of select="variable/xpath/@fullpath"/>
					</xsl:attribute>
					<xsl:attribute name="relativepath" namespace="">
						<xsl:value-of select="variable/xpath/@relativepath"/>
					</xsl:attribute>
					<xsl:attribute name="filter" namespace="">
						<xsl:value-of select="xpath/@filter"/>
					</xsl:attribute>
					<xsl:attribute name="namespace" namespace="">
						<xsl:value-of select="xpath/@namespace"/>
					</xsl:attribute>
					<xsl:attribute name="prefix" namespace="">
						<xsl:value-of select="xpath/@prefix"/>
					</xsl:attribute>
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="xpath/@type"/>
					</xsl:attribute>
					<xsl:attribute name="uri" namespace="">
						<xsl:value-of select="variable/xpath/@uri"/>
					</xsl:attribute>
				</xsl:element>
			</xsl:when>
															<!-- expressions (where) children are either  xpths or rawtext -->
															<!-- like above + resolveexpressionpaths-->
			<xsl:when test="count(orderby)=0 and count(return)=1 and count(variable[@type=&apos;let&apos;])=0 and count(variable[@type=&apos;for&apos;])=1 and  variable/xpath/@fullpath = return/xpath/@fullpath  and (count(expression/*)=count(expression/xpath)+count(expression/rawtext)  )">
				<xsl:element name="xpath" namespace="">
					<xsl:attribute name="fullpath" namespace="">
						<xsl:value-of select="variable/xpath/@fullpath"/>
					</xsl:attribute>
					<xsl:attribute name="relativepath" namespace="">
						<xsl:value-of select="variable/xpath/@relativepath"/>
					</xsl:attribute>
					<xsl:attribute name="filter" namespace="">
						<xsl:value-of select="variable/xpath/@filter"/>
					</xsl:attribute>
					<xsl:attribute name="namespace" namespace="">
						<xsl:value-of select="variable/xpath/@namespace"/>
					</xsl:attribute>
					<xsl:attribute name="prefix" namespace="">
						<xsl:value-of select="variable/xpath/@prefix"/>
					</xsl:attribute>
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="variable/xpath/@type"/>
					</xsl:attribute>
					<xsl:attribute name="uri" namespace="">
						<xsl:value-of select="variable/xpath/@uri"/>
					</xsl:attribute>
					<xsl:call-template name="resolveexpressionpaths"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
															<!-- otherwise reopen a statement and take it like ist is -->
				<xsl:element name="statement" namespace="">
															<!-- recursive descend -->
					<xsl:attribute name="type" namespace="">
						<xsl:value-of select="@type"/>
					</xsl:attribute>
					<xsl:call-template name="dochildren"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
<!-- ******************************************************************************* -->
	<xsl:template name="resolveexpressionpaths">
		<xsl:variable name="expressioncount" select="count(expression)"/>
		<xsl:for-each select="expression">
			<xsl:element name="path" namespace="">
															<!-- path == predicate -->
				<xsl:attribute name="closeparanth" namespace="">
					<xsl:value-of select="@closeparanth"/>
				</xsl:attribute>
				<xsl:attribute name="conjunction" namespace="">
					<xsl:value-of select="@conjunction"/>
				</xsl:attribute>
				<xsl:attribute name="openparanth" namespace="">
					<xsl:value-of select="@openparanth"/>
				</xsl:attribute>
				<xsl:attribute name="operation" namespace="">
					<xsl:value-of select="@operation"/>
				</xsl:attribute>
				<xsl:for-each select="*">
															<!-- those are XPath or RawText children only-->
					<xsl:choose>
						<xsl:when test="local-name()=&apos;xpath&apos;">
							<xsl:call-template name="doxpath"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:element name="rawtext" namespace="">
								<xsl:value-of disable-output-escaping="yes" select="text()"/>
							</xsl:element>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>
</xsl:stylesheet>
